'''
Created on May 16, 2012

@author: pascale
'''
# -*- coding: utf-8 -*-

try :    
    import matplotlib
    matplotlib.use('wxagg')
except :
    import sys
    sys.stderr.write('ERROR : cannot start rttovgui : check your python installation\n')                     
    sys.stderr.write('ERROR : matplotlib is not installed or is installed with an incorrect backend (wxagg is required)\n')   
    sys.exit(1)

import rmodel

import rttov
import sys
from util import *
from surfacectrl import *
from optionctrl import *
from profilectrl import *
import logging
import os
       
class MainController (GenericController) :
    """ Main Controller of the application """

    
    def __init__(self,app,debug):
        """ Constructor of the MainController
            define a Project object
            define the MainView Object
            define the undo redo Object
            make the binding with the different view
        
        """
        super(GenericController, self).__init__()

        GenericController.project=rmodel.project.Project()
        self.controlerName="MainController"
       
        #%(pathname)s        Full pathname of the source file where the logging
        #%(filename)s        Filename portion of pathname
        #%(module)s          Module (name portion of filename)
        #%(lineno)d          Source line number where the logging call was issued
        
        if debug : 
            level_logging=logging.DEBUG
        else :
            level_logging=logging.INFO
                
        logging.basicConfig(filename=self.project.config.ENV['GUI_WRK_DIR']+"/rttovgui.log" ,
                            format="[%(asctime)s] %(levelname)s [%(module)s:%(funcName)s:%(lineno)d] %(message)s",
                            level=level_logging,
                            datefmt="%Y:%m:%d %H:%M:%S",
                            filemode="w")       
        logging.info("start main controller")
        # set up the Mainview
        self.mv=rview.console.MainView(None)
   
        self.optionController=OptionController(self.mv)  
        self.chooseCoeffsController=ChooseCoeffsController(self.mv,self)
        self.surfaceController=SurfaceController(self.mv)
        self.profileController=ProfileController(self.mv)
        # set the binding 
        self.MakeBinding()
        self.rv=None 
        self.pcview=None
        self.kpcview=None
        self.kvpc=[]
        self.kv=[]
        self.RunkNumber=0
        pub.subscribe(self.ProfileLoadedListener,"Profile LOADED")
        pub.subscribe(self.CoeffsLoadedListener,"Coefficients CHANGED")
        self.write( "Welcome  ! ")

             
    def ProfileLoadedListener(self,msg):  
        self.prepLog()   
        logging.debug( "mainController ProfileLoadedListener")
        self.mv.EnableMenuItem('profileWindow')  
        self.mv.EnableMenuItem('optionsWindow')
        self.mv.EnableMenuItem('surfaceWindow')
        if self.project.myCoeffs.loadCoeffs :
            self.mv.EnableMenuItem('runDirect')  
            self.mv.EnableMenuItem('runK')
        
    def CoeffsLoadedListener(self,msg):    
        """ must be called when the coefficient are loaded or dropped 
        enable or disable menu rttov                              """
        self.prepLog()
        if self.project.myCoeffs.loadCoeffs == True :
            if self.project.loadProfile : 
                self.mv.EnableMenuItem('runDirect')  
                self.mv.EnableMenuItem('runK')
        else :
            self.mv.DisableMenuItem('runDirect')
            self.mv.DisableMenuItem('runK')    
            
    def SaveTheProfileAs(self,e):    
        """ apply profile change and save profile+option with a new name """
        self.prepLog()     
        # re-initialize the name of the saved profile file to None
        self.project.savedProfileFileName=None
        self.SaveTheProfile(e)    
        
    def MakeBinding(self) :
        """  Define the different binding of the application  """ 
        # binding for the MaineView (mv)
        self.mv.Bind(wx.EVT_MENU, self.OpenProfile, self.mv.items["openProfile"] )
        self.mv.Bind(wx.EVT_MENU, self.OpenAsciiProfile, self.mv.items["openAsciiProfile"] )
        self.mv.Bind(wx.EVT_MENU, self.mv.OnLoadCoefficients, self.mv.items["loadCoefficients"] )      
        self.mv.Bind(wx.EVT_MENU, self.SaveTheProfile,self.mv.items["saveProfile"] )
        self.mv.Bind(wx.EVT_MENU, self.optionController.OnOptions,self.mv.items["optionsWindow"] )
        self.mv.Bind(wx.EVT_MENU, self.surfaceController.OnSurface,self.mv.items["surfaceWindow"] )
        self.mv.Bind(wx.EVT_MENU, self.profileController.OnProfile,self.mv.items["profileWindow"] )
        self.mv.Bind(wx.EVT_MENU, self.SaveTheProfileAs,self.mv.items["saveProfileAs"] )
        self.mv.Bind(wx.EVT_MENU, self.LoadCoefficients,self.mv.items["loadCoefficients"])
        self.mv.Bind(wx.EVT_MENU, self.RunDirect,self.mv.items["runDirect"])
        self.mv.Bind(wx.EVT_MENU, self.RunK,self.mv.items["runK"])
        
    def SaveTheProfile(self,e):  self.SaveProfile(e,self.mv)     
       
    def RunDirect(self,e):
        """ Run direct model, if project.isPC() run direct PC"""
        self.prepLog() 
        if self.project.isPC():
            self._RunDirectPC(e)
        else :
            self._RunDirect(e)

            
    def _RunDirectPC(self,e):
        """ run direct model with PC """            
        if self.project.myOption["ADDSOLAR"] : 
            addsolar=1
        else : 
            addsolar=0
        # ask number or pc :
        if self.project.myCoeffs.loadCoeffs and self.project.isPC() :    
            if  self.project.myProfile['SKIN']['SURFTYPE'] != 1 :
                dialog=self.mv.WarmError("Use of principal components not allowed over a sea ice or a land surface")
                return                        
            dialog=rview.util.SimpleFSDialogBox( self.mv,-1 , "PC SCORES", text="Enter the number of Principal Components",text2="number",helptext="",minval=1,maxval=400,increment=1,defval=200 )
            dialog.CentreOnParent()
            if (dialog.ShowModal() == wx.ID_OK):
                n = dialog.GetSelections()    
                self.write('number of Principal Components '+str(n))
                self.project.npcscores=n
            else :
                return 
            self.mv.BeginBusy()
            err=self.project.runPC()
            self.mv.EndBusy() 
            self.write("RunDirectPC return code = "+str(err))
            if err == 0 :
                if self.pcview :
                    if self.pcview.IsIconized() : 
                        self.pcview.Restore()
                    self.pcview.ReRead(self.project.pcFileName)
                    self.pcview.Show()
                else :    
                    self.pcview=rview.pcView.pcView(self.mv,self.project.pcFileName)
                if self.project.myOption["ADDRADREC"] :
                    if self.rv:
                        if self.rv.IsIconized() : 
                            self.rv.Restore()
                        self.rv.read_rad_h5(self.project.pcFileName,addsolar)                     
                        self.rv.Refresh()
                    else: 
                        self.rv= rview.radianceframe.RadianceFrame(self.mv,"Radiance Viewer",self.project.pcFileName,addsolar)
                        self.rv.Show()
                    if self.surfaceController.surfaceView :  # a tester il y a peut-etre pas besoin si listener surface changed utilise
                        self.write( "plot emissivity reflectance")
                        self.surfaceController.surfaceView.PlotReflectanceEmissivity(self.project.myEmissivity,self.project.myReflectance)                        
                
    def _RunDirect(self,e):
        """ run the direct model if coefficient are loaded and a profile saved"""
        logging.debug("_RunDirect")
        if self.project.myOption["ADDSOLAR"] : 
            addsolar=1
        else : 
            addsolar=0
        if self.project.myCoeffs.loadCoeffs :
                self.mv.BeginBusy() 
                err=self.project.runDirect()
                self.mv.EndBusy() 
                if err == 0 :
                    if self.rv:
                        self.rv.read_rad_h5(self.project.radianceFileName,addsolar)                     
                        self.rv.Refresh()
                    else: 
                        self.rv= rview.radianceframe.RadianceFrame(self.mv,"Radiance Viewer",self.project.radianceFileName,addsolar)
                        self.rv.Show()
                    if self.surfaceController.surfaceView :  # a tester il y a peut-etre pas besoin si listener surface changed utilise
                        self.write( "plot emissivity reflectance")
                        self.surfaceController.surfaceView.PlotReflectanceEmissivity(self.project.myEmissivity,self.project.myReflectance)                        
        else  :
            self.mv.WarmError("You must load a Coefficients File ")
                         
    def RunK(self,e):

        self.prepLog()
        if self.project.isPC():
            self._RunKPC(e)
        else :
            self._RunK(e)

            
    
    
    def _RunK(self,e):
        """ run the K model if coefficient are loaded and a profile saved"""
        self.RunkNumber=self.RunkNumber+1
        if self.project.myCoeffs.loadCoeffs :
                self.mv.BeginBusy()
                err=self.project.runK()
                self.mv.EndBusy()
                if err == 0 :
                    self.write( "RunK successful")
                    kv=rview.kmatrixframe.KMatrixFrame(self.mv,"K Matrix Viewer run "+str(self.RunkNumber),self.project.KMatrixFileName,self.RunkNumber)
                    self.kv.append(kv)
                    kv.Show() 
                else :
                    logging.info("self.project.runK Failed")
                    self.mv.WarmError("RunK Failed ")                     
        else  :
            self.mv.WarmError("You must load a Coefficients File ")
                         
    def _RunKPC(self,e):
        """ run the K PC model if coefficient are loaded and a profile saved"""
        if self.project.myCoeffs.loadCoeffs and self.project.isPC() :    
            if  self.project.myProfile['SKIN']['SURFTYPE'] != 1 :
                dialog=self.mv.WarmError("Use of principal components not allowed over a sea ice or a land surface")
                return    
            dialog=rview.util.SimpleFSDialogBox( self.mv,-1 , "PC SCORES", text="Enter the number of Principal Components",text2="number",helptext="",minval=1,maxval=400,increment=1,defval=200 )
            dialog.CentreOnParent()
            if (dialog.ShowModal() == wx.ID_OK):
                n = dialog.GetSelections()    
                self.write('number of Principal Components  '+str(n))
                self.project.npcscores=n
            else :
                return 
            self.mv.BeginBusy()   
            err=self.project.runPCK()
            self.mv.EndBusy() 
            if err == 0 :
                self.write( "Run PC K successful")
                self.RunkNumber+=1
                fileName=self.project.pcKMatrixFileName
                print "fileName:",fileName,"create kpcmatrixframe "
                if self.project.myOption["ADDRADREC"] :
                    kv=rview.kmatrixframe.KMatrixFrame(self.mv,"K Matrix Viewer run "+str(self.RunkNumber),fileName,self.RunkNumber)
                    self.kv.append(kv)
                    kv.Show()
                else :     
                    kvpc=rview.kpcmatrixframe.KPCMatrixFrame(self.mv,"K Matrix Viewer run "+str(self.RunkNumber),fileName,self.RunkNumber)
                    self.kvpc.append(kvpc)
                    kvpc.Show()
                    #kvpc.CenterOnParent() 
                    kpcprofile=rview.kpcView.kpcView(self.mv,None,None,0,10,fileName,run=self.RunkNumber)
                    kpcprofile.Show()
            else :
                self.mv.WarmError("RunK Failed ")                     
        else  :
            self.mv.WarmError("You must load a Coefficients File ")
 
    def OpenAsciiProfile(self,e):
        """ open an ascii profile """
        self.prepLog()
        profile_ascii=self.mv.OnOpenFile(e,self.project.config.ENV['RTTOV_GUI_PREFIX']+"/profile-datasets-py")
        if (profile_ascii):
            try :
                self.project.openAsciiProfile( profile_ascii)
                self.write("Open ascii profile : "+ str(profile_ascii))
            except IOError : 
                self.write( "Error while opening "+ str(profile_ascii) )   
                
    def OpenProfile(self,e):
        """ Open the profile , if it contains more than one profile ask for the number of the profile """
        self.prepLog()
        self.profileFileName=self.mv.OnOpenFile(e,self.project.config.ENV['RTTOV_GUI_PROFILE_DIR'])
        self.write( "Open profile filename= "+ str(self.profileFileName)  )     
        if (self.profileFileName ):
            try :
                number=rttov.profile.getNumberOfProfiles(self.profileFileName)  
                self.write( "number of profile n this file : "+str(number))
                if ( number > 1 ) :
                    number=self.mv.ChooseNumber (number,"select a profile number","profile number selection")
                self.project.openProfile(self.profileFileName,number)
            except IOError : 
                self.write( "Error while opening "+ str(self.profileFileName) )   # ne marche pas pb h5py erreur non remontee

    def LoadCoefficients(self,e):
        """ open the chooseCoefsController windows """    
        self.prepLog()
        self.chooseCoeffsController.OnChoose(e)
       
                
         
    
class ChooseCoeffsController (GenericController) :
    """ controller for the Option windows """
    def __init__(self,parentFrame,parentController):
        """ Constructor of the MainController
            define a Project object
            create the OptionView Object
            make the binding with the options view
        
        """        
        self.theParentFrame=parentFrame
        self.theParentController=parentController
        super(GenericController, self).__init__()
        self.chooseCoeffsWindow=None
        self.controlerName="ChooseCoeffsController"
        
    def OnChoose(self,e):
        """ create the choose coefficients file window and make the binding of the items of this
        window with the ChooseCoeffsControlle methods """
        self.chooseCoeffsWindow=rview.coeff.CoeffFilesView(self.theParentFrame,self.project.myCoeffs)
        self.chooseCoeffsWindow.Bind(wx.EVT_MENU, self.OnApplyCoefficients,self.chooseCoeffsWindow.items["applyCoefficients"] )
        self.chooseCoeffsWindow.Bind(wx.EVT_MENU, self.OnLoadCoefficients,self.chooseCoeffsWindow.items["loadCoefficients"] )
        self.chooseCoeffsWindow.Bind(wx.EVT_BUTTON, self.OnApplyCoefficients,self.chooseCoeffsWindow.applyBtn )
        self.chooseCoeffsWindow.Bind(wx.EVT_BUTTON, self.OnLoadCoefficients,self.chooseCoeffsWindow.loadBtn )
        self.chooseCoeffsWindow.Bind(wx.EVT_BUTTON, self.OnDropCoefficients,self.chooseCoeffsWindow.dropBtn )

    def OnLoadCoefficients(self,e):
        """ save selected name in project object and try to load coefficients """
        self.project.myCoeffs=self.chooseCoeffsWindow.myCoeffs
        if (self.project.myCoeffs.fileName["standard"] == "" ) :
            self.chooseCoeffsWindow.WarmError("you must choose the coefficients files")
        else :     
            self.write( "loadCoefficients "+ str(self.project.myCoeffs.fileName))
            try :
                self.chooseCoeffsWindow.BeginBusy()
                returnCode=self.project.loadCoefficients()
                if (returnCode == 0 ) : 
                    self.write( "Coefficients successfully loaded")
                    self.chooseCoeffsWindow.EndBusy()
                    self.chooseCoeffsWindow.ShowInfoMessage("Coefficients successfully loaded")
                    self.chooseCoeffsWindow.Close()
                else :
                    self.chooseCoeffsWindow.EndBusy()
                    self.chooseCoeffsWindow.WarmError("Can't Load coefficients : \n \
                    files are not rttov coefficient files or \n \
                    check file names or compatibility issue \n \
                    between standard  clouds  aerosols and pc  ")

                
            except IOError : 
                self.write( "Error while opening "+ self.project.myCoeffs.fileName)
        
    def OnApplyCoefficients(self,e):
        """ save selected name in project object """
        self.project.myCoeffs=self.chooseCoeffsWindow.myCoeffs
        

    def OnDropCoefficients(self,e):
        """ drop the coeeficients """
        ret=self.project.dropCoefficients()
        if ret == 0 :
            self.chooseCoeffsWindow.OnClear(e)
            self.project.myCoeffs=self.chooseCoeffsWindow.myCoeffs
        else :
            pass        

def is_valid_file(parser, arg):
    if not os.path.exists(arg):
        parser.error("The file %s does not exist!"%arg)
    else:
        return open(arg,'r')  #return an open file handle    
   
def check_prerequisite():
    
    python_version=sys.version

    matplotlib_version=matplotlib.__version__
    if wx.VERSION[0]== 2 and wx.VERSION[1]== 8 :
        sys.stderr.write('WARNING: wx python version = 2.8.x  some features of RTTOVGUI might not work properly\n')
    if  matplotlib_version < "1.3.1"  :
        sys.stderr.write('WARNING: matplotlib version < 1.3.1 some features of RTTOVGUI might not work properly\n')
    import h5py.version
    if h5py.version.version_tuple[0] < 2 :
        sys.stderr.write('WARNING: h5py version < 2.0.1 some features of RTTOVGUI might not work properly\n')
        
def main():
    # import cProfile
    import argparse
     
    check_prerequisite() 
    
    app=wx.App(False)


    parser = argparse.ArgumentParser(description="launch RTTOV GUI", conflict_handler='resolve')
    parser.add_argument('-P', '--profile',  dest="profile",  help="input file name",    required=False,
                        metavar="FILE",    type=lambda x: is_valid_file(parser,x))
    parser.add_argument('-s', '--std-coeff',  dest="stdcoeff",  help="standard coefficient file", required=False,
                        metavar="FILE",  type=lambda x: is_valid_file(parser,x))
    parser.add_argument('-c', '--cloud-coeff', dest="cldcoeff", help="clouds coefficient file",  required=False,
                        metavar="FILE",    type=lambda x: is_valid_file(parser,x))
    parser.add_argument('-a', '--aer-coeff', dest="aercoeff", help="aerosols coefficient file ", required=False,
                        metavar="FILE",    type=lambda x: is_valid_file(parser,x))
    parser.add_argument('-p', '--pc-coeff', dest="pccoeff", help="PC coefficient file ",   required=False,
                        metavar="FILE",    type=lambda x: is_valid_file(parser,x))
    parser.add_argument("-d", "--debug",     dest="debug", action="store_true",       help="debug", required=False)
    

    args = parser.parse_args()
     
    controller=MainController(app,args.debug)
    python_version=sys.version
    wx_version=wx.__version__
    matplotlib_version=matplotlib.__version__
    logging.info("*******   RTTOVGUI environment       *********************")
    logging.info("Python version :"+ python_version)
    logging.info('wxPython version :'+ wx_version)
    logging.info('matplotlib version :'+ matplotlib_version)
    logging.info("PYTHONPATH : "+str(os.environ["PYTHONPATH"]))
    logging.info("LD_LIBRARY_PATH : "+str(os.environ["LD_LIBRARY_PATH"]))
    sys.stderr.write(str(controller.project.config.ENV)+"\n")
    logging.info(str(controller.project.config.ENV))  
    logging.info("*********************************************************")  
    sys.stderr.write("**********************************************************************\n")
    
    
    sys.stderr.write("Start RTTOVGUI ...\n")    
    if args.profile != None : 
        print type(args.profile)
        controller.profileFileName =args.profile.name
        sys.stderr.write("... load profile"+controller.profileFileName+"\n" )
        logging.info("profile:"+controller.profileFileName) 
     
        try :
            print  "profile:"+controller.profileFileName
            err=controller.project.openProfile(controller.profileFileName,1)
            if err != 0 : 
                sys.stderr.write("ERROR wrong profile file \n")
                logging.info("wrong profile names ")
                sys.exit(1)
        except  : 
            
            sys.stderr.write( "ERROR wrong profile names \n") 
            logging.info("wrong profile names ")
            sys.exit(1)      
    if args.cldcoeff != None : controller.project.myCoeffs.fileName["clouds"] = args.cldcoeff.name
    if args.aercoeff != None : controller.project.myCoeffs.fileName["aerosols"] = args.aercoeff.name
    if args.pccoeff != None : controller.project.myCoeffs.fileName["PC"] = args.pccoeff.name
    if args.stdcoeff  != None :  
        controller.project.myCoeffs.fileName["standard"] = args.stdcoeff.name
        sys.stderr.write("... load coefficient"+str(controller.project.myCoeffs.fileName)+"\n")
        logging.info (" load coefficients")
        logging.info (str(controller.project.myCoeffs.fileName))

        err= controller.project.loadCoefficients()
        if err != 0 : 
            print "ERROR wrong coefficient names "
            logging.info("wrong coefficient names ")
            sys.exit(1)
    logging.info("start MainLoop")

    app.MainLoop()
   # cProfile.run('app.MainLoop()')
if __name__ == "__main__":
    main()
