SUBROUTINE rttov_alloc_opt_param( &
            & err,       &
            & opt_param, &
            & nchanprof, &
            & nlayers,   &
            & nphangle,  &
            & asw)
! Description:
!   Allocates/deallocates the user optical parameter structure
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2012, EUMETSAT, All Rights Reserved.
!
! Method:
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date     Comment
! -------   ----     -------
! 1.0       14/09/12 Created (J Hocking)
!
!
! Code Description:
!   Language:           Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".

!INTF_OFF
#include "throw.h"
!INTF_ON
  USE parkind1, ONLY : jpim
  USE rttov_types, ONLY : rttov_opt_param

  IMPLICIT NONE
  INTEGER(KIND=jpim),    INTENT(INOUT)          :: err
  TYPE(rttov_opt_param), INTENT(INOUT)          :: opt_param
  INTEGER(KIND=jpim),    INTENT(IN)             :: nchanprof
  INTEGER(KIND=jpim),    INTENT(IN)             :: nlayers
  INTEGER(KIND=jpim),    INTENT(IN)             :: nphangle
  INTEGER(KIND=jpim),    INTENT(IN)             :: asw
!INTF_END
#include "rttov_errorreport.interface"

  TRY

  IF (asw .EQ. 1) THEN
    NULLIFY (opt_param%abs,      &
             opt_param%sca,      &
             opt_param%bpr,      &
             opt_param%pha,      &
             opt_param%phangle,  &
             opt_param%iphangle, &
             opt_param%cosphangle)
    ALLOCATE (opt_param%abs(nchanprof, nlayers),           &
              opt_param%sca(nchanprof, nlayers),           &
              opt_param%bpr(nchanprof, nlayers),           &
              opt_param%pha(nchanprof, nlayers, nphangle), &
              opt_param%phangle(nphangle),                 &
              STAT = err) !iphangle and cosphangle are allocated in init routine
    THROWM(err.ne.0,"Allocation of opt_param failed")
  ENDIF
  IF (asw .EQ. 0) THEN
    DEALLOCATE (opt_param%abs,        &
                opt_param%sca,        &
                opt_param%bpr,        &
                opt_param%pha,        &
                opt_param%phangle,    &
                STAT = err)
    THROWM(err.ne.0,"Deallocation of opt_param failed")
    IF (ASSOCIATED(opt_param%iphangle))   DEALLOCATE(opt_param%iphangle, STAT = err)
    THROWM(err.ne.0,"Deallocation of opt_param%iphangle failed")
    IF (ASSOCIATED(opt_param%cosphangle)) DEALLOCATE(opt_param%cosphangle, STAT = err)
    THROWM(err.ne.0,"Deallocation of opt_param%cosphangle failed")
    NULLIFY (opt_param%abs,      &
             opt_param%sca,      &
             opt_param%bpr,      &
             opt_param%pha,      &
             opt_param%phangle,  &
             opt_param%iphangle, &
             opt_param%cosphangle)
  ENDIF

  CATCH

END SUBROUTINE rttov_alloc_opt_param
